%imgopen	Laura Santos Campis 11/7/99
%loading of remote sensing images, Bip,Bil,Bsq formats
%Band Interleaved by Pixel BIP (or Sample BIS)
%Band Interleaved by Line BIL
%Band Sequencial BSQ
%subgroups of lines and columns for selected bands

function y = imgopen(filename,format,bands,lines,columns,pixelsize,b2u,l1,l2,c1,c2)
%filename: name of image
%format: image format BIP, BIL o BSQ
%bands: number of bands
%lines: number of lines
%columns: number of columns
%pixelsize: number from 0 to 13 indicating the pixel precision
%b2u vector: contains the numbers of the bands to be used
%l1 : number of first line to be used
%l2 : number of last line to be used
%c1 : number of first column to be used
%c2 : number of last column to be used

fid = fopen(filename,'r','s');

%Determine psize: pixel precision
%nb: number of bytes for the precision
switch(pixelsize)   
case 1           
   psize = ('char'); 
   nb = 1;
case 2
   psize = ('uchar');
   nb = 1;
case 3
   psize = ('schar');
   nb = 1;
case 4
   psize = ('int8');
   nb = 1;
case 5
   psize = ('int16');
   nb = 2;
case 6
   psize = ('int32');
   nb = 4;
case 7
   psize = ('int64');
   nb = 8;
case 8
   psize = ('uint8');
   nb = 1;
case 9
   psize = ('uint16');
   nb = 2;
case 10
   psize = ('uint32');
   nb = 4;
case 11
   psize = ('uint64');
   nb = 8;
case 12
   psize = ('float32');
   nb = 4;
case 13
   psize = ('float64');
   nb = 8;
case 14
   psize = ('double');
   nb = 8;
end  

%vector containing the range of lines to be used
l2u=[];
for i=l1:l2,
   l2u=[l2u,i];
end
%number of selected lines
slines= l2-l1+1;
%number of selected columns
scolumns = c2-c1+1;
%dimensions for the output matrix
newcolumns = slines*scolumns;
newlines = size(b2u,2);

switch(format)
case 1
   %BIP (BIS)
   %Band Interleaved by Pixel (or Sample) BIP (BIS)
   %offsets to the first element of every selected band
	b=(b2u-1)*nb;
   %offsets to the first element of every selected line
	l=(l2u-1)*bands*columns*nb;
	%offset to the element of the first selected column
	c=(c1-1)*bands*nb;
	%offset between elements of the same band
	offband=(bands-1)*nb;
   
   bip = zeros(newlines,newcolumns);
   
   for i = 1:newlines,
   	colmn = 0;
   	fseek(fid,b(1,i),'bof');
   	for j= 1:slines,
      	fseek(fid,l(1,j),'cof');
      	fseek(fid,c,'cof');
      	for k= 1:scolumns,
         	bip(i,k+colmn) = fread(fid,1,psize);
         	fseek(fid,offband,'cof');
      	end
      	colmn = colmn+scolumns;
      	fseek(fid,b(1,i),'bof');
   	end
	end
	fclose(fid);
	y=bip;

case 2
   %BIL
   %Band Interleaved by Line BIL
   %offsets to the first element of every selected band
	b=(b2u-1)*columns*nb;
   %offsets to the first element of every selected line
   l=(l2u-1)*bands*columns*nb;
	%offset to the element of the first selected column
	c=(c1-1)*nb;
   
   bil=zeros(newcolumns,newlines);
   
   for i = 1:newlines,
   k = 1;
   fseek(fid,b(1,i),'bof');
 	  for j = 1:slines,
   	   fseek(fid,l(1,j),'cof');
      	fseek(fid,c,'cof');
      	temp=fread(fid,scolumns,psize); 
      	bil(k:k+scolumns-1,i) = temp;
         k= k+scolumns;
      	fseek(fid,b(1,i),'bof');
   	end
	end
	fclose(fid);
	y=bil';   

case 3
   %BSQ
   %Band Sequencial BSQ
   %offsets to the first element of every selected band
	b=(b2u-1)*lines*columns*nb;
   %offsets to the first element of every selected line
	l=(l2u-1)*columns*nb;
	%offset to the element of the first selected column
	c=(c1-1)*nb;

	bsq=zeros(newcolumns,newlines);
   
   for i= 1:newlines,
   	k = 1;
   	fseek(fid,b(1,i),'bof');
   	for j = 1:slines,
      	fseek(fid,l(1,j),'cof');
      	fseek(fid,c,'cof');
      	temp=fread(fid,scolumns,psize);
      	bsq(k:k+scolumns-1,i) = temp;
      	k = k+scolumns;
      	fseek(fid,b(1,i),'bof');
   	end
	end
	fclose(fid);
	y=bsq';
   
end
